@@ -1,12 +1,12 @@
-;;; find-file-in-project.el --- Find file/directory and review Diff/Patch/Commit efficiently everywhere
+;;; find-file-in-project.el --- Find file/directory and review Diff/Patch/Commit efficiently -*- coding: utf-8 -*-
 
 ;; Copyright (C) 2006-2009, 2011-2012, 2015-2018
 ;;   Phil Hagelberg, Doug Alcorn, Will Farrington, Chen Bin
 ;;
-;; Version: 6.0.7
+;; Version: 6.2.0
 ;; Author: Phil Hagelberg, Doug Alcorn, and Will Farrington
 ;; Maintainer: Chen Bin <chenbin.sh@gmail.com>
-;; URL: https://github.com/technomancy/find-file-in-project
+;; URL: https://github.com/redguardtoo/find-file-in-project
 ;; Package-Requires: ((emacs "25.1"))
 ;; Created: 2008-03-18
 ;; Keywords: project, convenience
@@ -111,19 +111,31 @@
 ;;
 ;; `ffip-fix-file-path-at-point' replaces path at point with correct relative/absolute path.
 ;;
+;; File/directory searching actions are automatically stored into `ffip-find-files-history'.
+;; Use `ffip-find-files-resume' to replay any previous action.
+;; The maximum number of items of the history is set in `ffip-find-files-history-max-items'.
+;;
 ;; `ffip-show-diff' execute the backend from `ffip-diff-backends'.
 ;; The output is in Unified Diff Format and inserted into *ffip-diff* buffer.
 ;; Press "o" or "C-c C-c" or "ENTER" or `M-x ffip-diff-find-file' in the
-;; buffer to open corresponding file.
+;; buffer to open corresponding file.  Please note some backends assume that the git cli program
+;; is added into environment variable PATH.
 ;;
-;; `ffip-diff-find-file-before-hook' is called before `ffip-diff-find-file'.
+;; `ffip-diff-find-file-before-hook' is called in `ffip-diff-find-file'.
+;; Two file names are passed to it as parameters.  One name is returned by the hook
+;; as the file searching keyword.
 ;;
 ;; `ffip-diff-apply-hunk' applies current hunk in `diff-mode' (please note
 ;; `ffip-diff-mode' inherits from `diff-mode') to the target.
 ;; file. The target file could be located by searching `recentf-list'.
 ;; Except this extra feature, `ffip-diff-apply-hunk' is same as `diff-apply-hunk'.
 ;; So `diff-apply-hunk' can be replaced by `ffip-diff-apply-hunk'.
-
+;;
+;; `ffip-diff-filter-hunks-by-file-name' can filter hunks by their file names.
+;; User input pattern "regex !exclude1 exclude1" means the hunk's file name does match "regex".
+;; But does not match "exclude1" or "exclude2".;
+;; Please note in "regex", space represents any string.
+;;
 ;; If you use `evil-mode', insert below code into ~/.emacs,
 ;;   (defun ffip-diff-mode-hook-setup ()
 ;;       (evil-local-set-key 'normal "K" 'diff-hunk-prev)
@@ -149,10 +161,11 @@
 ;;
 ;; This program works on Windows/Cygwin/Linux/macOS
 ;;
-;; See https://github.com/technomancy/find-file-in-project for advanced tips.
+;; See https://github.com/redguardtoo/find-file-in-project for advanced tips.
 
 ;;; Code:
 
+(require 'find-file)
 (require 'find-lisp)
 (require 'diff-mode)
 (require 'windmove)
@@ -178,6 +191,12 @@
   :type 'boolean
   :safe #'booleanp)
 
+(defcustom ffip-project-search-function 'ffip-project-search-default-function
+  "Function to execute find program in shell."
+  :group 'ffip
+  :type 'function
+  :safe #'functionp)
+
 (defcustom ffip-prefer-ido-mode nil
   "Prefer `ido-completing-read' to filter file candidates."
   :group 'ffip
@@ -189,7 +208,11 @@
   :group 'ffip
   :type 'string)
 
-(defvar ffip-filename-history nil)
+(defcustom ffip-diff-find-file-by-file-name-p nil
+  "When find file in diff hunk, only search by file name."
+  :group 'ffip
+  :type 'boolean
+  :safe #'booleanp)
 
 (defcustom ffip-strip-file-name-regex
   "\\(\\.mock\\|\\.test\\|\\.mockup\\)"
@@ -198,6 +221,18 @@ It's used by `find-file-with-similar-name'."
   :group 'ffip
   :type 'regexp)
 
+(defcustom ffip-find-files-history-max-items 4
+  "Maximum number of items stored in `ffip-find-files-history'.
+If this number is zero, no item is saved into `ffip-find-files-history'."
+  :group 'ffip
+  :type 'integer)
+
+(defvar ffip-filename-history nil
+  "History of file names provided by users.")
+
+(defvar ffip-find-files-history nil
+  "History generated by `ffip-find-files'.")
+
 (defvar ffip-diff-find-file-before-hook nil
   "Hook before `ffip-diff-find-file' move focus out of *ffip-diff* buffer.")
 
@@ -211,52 +246,76 @@ The file path is passed to the hook as the first argument.")
 (defvar ffip-relative-path-pattern "^\\(\\.\\.*/\\)+"
   "Pattern of relative path.")
 
-(defun ffip-shell-command-to-string (command)
-  "Execute shell COMMAND and return its output as a string."
-  (with-output-to-string
-    (with-current-buffer
-        standard-output
-      (shell-command command t))))
-
 (defun ffip-nonempty-lines (str)
   "Return non empty lines from STR."
   (split-string str "[\r\n]+" t))
 
 (defun ffip-diff-git-versions ()
   "List all versions of code under Git."
-  (let* ((git-cmd (concat "git --no-pager log --date=short --pretty=format:'%h|%ad|%s|%an' "
-                          buffer-file-name)))
-    (nconc (ffip-nonempty-lines (shell-command-to-string "git branch --no-color --all"))
-           (ffip-nonempty-lines (shell-command-to-string git-cmd)))))
+  (let* ((cmd1 "git branch --no-color --all")
+         (cmd2 (concat "git --no-pager log --date=short --pretty=format:'%h|%ad|%s|%an'" buffer-file-name)))
+    (nconc (ffip-nonempty-lines (shell-command-to-string cmd1))
+           (ffip-nonempty-lines (shell-command-to-string cmd2)))))
+
+(defun ffip-diff-select-version ()
+  "Select a version from git history."
+  (let* ((line (completing-read "Select from git history: " (ffip-diff-git-versions)))
+         (version (replace-regexp-in-string "^ *\\*? *"
+                                            ""
+                                            (car (split-string line "|" t)))))
+    version))
 
 ;;;###autoload
 (defun ffip-git-diff-current-file ()
-  "Run 'git diff version:current-file current-file'."
-  (let* ((default-directory (locate-dominating-file default-directory ".git"))
-         (line (completing-read "diff current file: " (ffip-diff-git-versions))))
-    (ffip-shell-command-to-string (format "git --no-pager diff %s:%s %s"
-                                     (replace-regexp-in-string "^ *\\*? *" "" (car (split-string line "|" t)))
+  "Compare another version of current file."
+  (let* ((default-directory (locate-dominating-file default-directory ".git")))
+    (shell-command-to-string (format "git --no-pager diff %s:%s %s"
+                                     (ffip-diff-select-version)
                                      (file-relative-name buffer-file-name default-directory)
                                      buffer-file-name))))
 
 (defun ffip-git-diff-project()
-  "Run 'git diff version' in project."
-  (let* ((default-directory (locate-dominating-file default-directory ".git"))
-         (line (completing-read "diff current file: " (ffip-diff-git-versions)))
-         (version (replace-regexp-in-string "^ *\\*? *" "" (car (split-string line "|" t)))))
-    (ffip-shell-command-to-string (format "git --no-pager diff %s" version))))
+  "Compare another version of project."
+  (let* ((default-directory (locate-dominating-file default-directory ".git")))
+    (shell-command-to-string (format "git --no-pager diff %s"
+                                     (ffip-diff-select-version)))))
+
+(defun ffip-git-diff-directory()
+  "Compare another version of current directory."
+  (when buffer-file-name
+    (let* ((dir (read-directory-name "Directory: " (file-name-directory buffer-file-name)))
+           (default-directory (locate-dominating-file default-directory ".git")))
+      (shell-command-to-string (format "git --no-pager diff %s -- \"%s\""
+                                       (ffip-diff-select-version)
+                                       dir)))))
+
+(defun ffip-git-diff-file-extension()
+  "Compare another version of files matching file extensions from user input.
+File extensions are separated by space character."
+  (let* ((patterns (read-string "File extensions (e.g., \"cpp py\" matches C++ and Python code files): "
+                                (and buffer-file-name (file-name-extension buffer-file-name)))))
+    (when patterns
+      (let* ((default-directory (locate-dominating-file default-directory ".git")))
+        (setq patterns
+              (mapconcat (lambda (s) (format "\"*.%s\"" s)) (split-string patterns " +") " "))
+        (shell-command-to-string (format "git --no-pager diff %s -- %s"
+                                         (ffip-diff-select-version)
+                                         patterns))))))
 
 (defvar ffip-diff-backends
   '(ffip-git-diff-current-file
     ffip-git-diff-project
+    ffip-git-diff-directory
+    ffip-git-diff-file-extension
     ("`git diff HEAD^` in project" . "cd $(git rev-parse --show-toplevel) && git diff HEAD^")
     ("`git diff --cached` in project" . "cd $(git rev-parse --show-toplevel) && git diff --cached")
     ("`git diff` in project" . "cd $(git rev-parse --show-toplevel) && git diff")
-    ("`git diff` current file" . (ffip-shell-command-to-string (format "cd $(git rev-parse --show-toplevel) && git diff '%s'"
+    ("`git diff` current file" . (shell-command-to-string (format "cd $(git rev-parse --show-toplevel) && git diff \"%s\""
                                                                     (buffer-file-name))))
-    ("`git log -p` current file" . (ffip-shell-command-to-string (format "cd $(git rev-parse --show-toplevel) && git --no-pager log --date=short -p '%s'"
+    ;; git option "--cc" shows changes in merge commits
+    ("`git log -p` current file" . (shell-command-to-string (format "cd $(git rev-parse --show-toplevel) && git --no-pager log --date=short -p --cc \"%s\""
                                                      (buffer-file-name))))
-    ("`git log -S keyword -p` in project" . (ffip-shell-command-to-string (format "cd $(git rev-parse --show-toplevel) && git --no-pager log --date=short -S'%s' -p"
+    ("`git log -S keyword -p` in project" . (shell-command-to-string (format "cd $(git rev-parse --show-toplevel) && git --no-pager log --date=short -S\"%s\" -p --cc"
                                                               (read-string "Git search string: "))))
     ("Diff from `kill-ring'" . (car kill-ring)))
   "The list of back-ends.
@@ -299,6 +358,8 @@ May be set using .dir-locals.el.  Checks each entry if set to a list."
     "*/.hg"
     "*/.DS_Store"
     "*/.sass-cache"
+    "*/elpy"
+    "*/dcache"
     "*/.npm"
     "*/.tmp"
     "*/.idea"
@@ -534,6 +595,7 @@ If CHECK-ONLY is true, only do the check."
          (i 0)
          j
          (dirs '(":\\\\cygwin64\\\\bin\\\\"
+                 ":\\\\cygwin-portable\\\\cygwin\\\\bin\\\\"
                  ":\\\\msys64\\\\usr\\\\bin\\\\"))
          rlt)
     (while (and (not rlt)
@@ -700,6 +762,11 @@ BSD/GNU Find use glob pattern."
   ;; also @see #15 improving handling of directories containing space
   `(push (cons (replace-regexp-in-string "^\./" "" ,file) ,file) ,result))
 
+(defun ffip-project-search-default-function (find-command)
+  "Execute FIND-COMMAND in shell and split its output into lines."
+  (if ffip-debug "ffip-project-search-default-function => find-command=%s" find-command)
+  (split-string (shell-command-to-string find-command) "[\r\n]+" t))
+
 ;;;###autoload
 (defun ffip-project-search (keyword &optional find-directory-p)
   "Return an alist of all filenames in the project and their path.
@@ -713,7 +780,7 @@ IF FIND-DIRECTORY-P is t, we are searching directories, else files."
   (let* ((default-directory (ffip-get-project-root-directory))
          (cmd (ffip-create-shell-command keyword find-directory-p))
          (fd-file-pattern (concat "^" (mapconcat 'ffip-glob-to-regex ffip-patterns "\\|") "$"))
-         (collection (split-string (ffip-shell-command-to-string cmd) "[\r\n]+" t))
+         (collection (funcall ffip-project-search-function cmd))
          rlt)
 
     (if ffip-debug (message "run command at %s: %s" default-directory cmd))
@@ -747,6 +814,29 @@ IF FIND-DIRECTORY-P is t, we are searching directories, else files."
     (format "Find in %s/: "
             (file-name-nondirectory (directory-file-name root)))))
 
+(defun ffip-select-and-open-file (files new-window-p directory-p fn lnum)
+  "Select and open file from FILES.
+If NEW-WINDOW-P is t, create a new window for opened file.
+If DIRECTORY-P is t, open directory instead of file.
+IF the function FN is not nil, call it after opening the selected file.
+After opening the file, forward LNUM lines."
+  (ffip-completing-read
+   (ffip-hint)
+   files
+   `(lambda (file)
+      ;; only one item in project files
+      (if ,directory-p
+          (if (quote ,new-window-p)
+              (dired-other-window file)
+            (switch-to-buffer (dired file)))
+        ;; open file
+        (if (quote ,new-window-p)
+            (find-file-other-window file)
+          (find-file file))
+        ;; goto line if needed
+        (ffip--forward-line ,lnum)
+        (if ,fn (funcall ,fn file))))))
+
 ;;;###autoload
 (defun ffip-find-files (keyword open-another-window &optional find-directory-p fn)
   "Use KEYWORD to find files.
@@ -763,25 +853,50 @@ This function is the API to find files."
     (setq cands (ffip-project-search keyword find-directory-p))
     (cond
      ((> (length cands) 0)
-      (ffip-completing-read
-       (ffip-hint)
-       cands
-       `(lambda (file)
-          ;; only one item in project files
-          (if ,find-directory-p
-              (if (quote ,open-another-window)
-                  (dired-other-window file)
-                (switch-to-buffer (dired file)))
-            ;; open file
-            (if (quote ,open-another-window)
-                (find-file-other-window file)
-              (find-file file))
-            ;; goto line if needed
-            (ffip--forward-line ,lnum)
-            (if ,fn (funcall ,fn file))))))
+
+      (unless (eq ffip-find-files-history-max-items 0)
+        ;; save the history
+        (when (>= (length ffip-find-files-history)
+                  ffip-find-files-history-max-items)
+          ;; kick out the oldest item
+          (setq ffip-find-files-history (butlast ffip-find-files-history)))
+        ;; add the latest item
+        (push (list :files cands
+                    :keyword keyword
+                    :directory-p find-directory-p
+                    :function fn
+                    :forward-lines lnum
+                    :default-directory (ffip-get-project-root-directory))
+              ffip-find-files-history))
+
+      (ffip-select-and-open-file cands
+                                 open-another-window
+                                 find-directory-p
+                                 fn
+                                 lnum))
+
      (t
       (message "Nothing found!")))))
 
+;;;###autoload
+(defun ffip-find-files-resume (&optional n)
+  "Resume the last Nth `ffip-find-file' operation.
+Please note N is zero originated."
+  (interactive "P")
+  (unless n (setq n 0))
+  (cond
+   ((>= n ffip-find-files-history-max-items)
+    (message "There are only %d items in `ffip-find-files-history'."
+             ffip-find-files-history-max-items))
+   (t
+    (let* ((item (nth n ffip-find-files-history))
+           (default-directory (plist-get item :default-directory)))
+      (ffip-select-and-open-file (plist-get item :files)
+                                 nil
+                                 (plist-get item :directory-p)
+                                 (plist-get item :function)
+                                 (plist-get item :forward-lines))))))
+
 (defun ffip--prepare-root-data-for-project-file (root)
   "Prepare data for ROOT."
   (cons 'ffip-project-root root))
@@ -871,11 +986,14 @@ You can override this by setting the variable `ffip-project-root'."
   (if (string-match-p ffip-relative-path-pattern filename) t))
 
 (defun ffip-guess-file-name-at-point ()
-  "Guess file name at point."
-  (or (and (region-active-p) (ffip--read-selected))
-      (thing-at-point 'filename)
-      (thing-at-point 'symbol)
-      (read-string "No file name at point. Please provide one: ")))
+  "Guess file name at point.  File name could contain environment variables."
+  (let* ((file (or (and (region-active-p) (ffip--read-selected))
+                   (thing-at-point 'filename)
+                   (thing-at-point 'symbol)
+                   (read-string "No file name at point. Please provide one: "))))
+    (when file
+      ;; replace environment variable in file name
+      (car (ff-list-replace-env-vars (list file))))))
 
 (defun ffip--guess-physical-path (file)
   "Return physical full path of FILE which does exist."
@@ -910,7 +1028,7 @@ You can override this by setting the variable `ffip-project-root'."
 
 ;;;###autoload
 (defun find-file-in-project-at-point (&optional open-another-window)
-  "Find file whose name is guessed around point.
+  "Find file at point.  File path could contain environment variables.
 If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
   (interactive "P")
   (let* ((fn (ffip-guess-file-name-at-point))
@@ -1097,6 +1215,7 @@ This command works in any environment (Windows, etc) out of box."
 
 ;;;###autoload
 (defalias 'ffip 'find-file-in-project)
+;;;###autoload
 (defalias 'find-relative-path 'ffip-find-relative-path)
 
 (defun ffip-path (candidate)
@@ -1120,7 +1239,8 @@ If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
          (alnum 0)
          (blnum 0)
          (regex "\\(?:\\*\\{15\\}.*\n\\)?[-@* ]*\\([0-9,]+\\)\\([ acd+]+\\([0-9,]+\\)\\)?")
-         (ffip-match-path-instead-of-filename t))
+         (ffip-match-path-instead-of-filename (not ffip-diff-find-file-by-file-name-p))
+         keyword)
 
     (save-excursion
       (diff-beginning-of-hunk t)
@@ -1131,24 +1251,37 @@ If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
     (cond
      ((or (null files) (eq (length files) 0))
       (message "No file is found!"))
+
+     ;; file A and file B have the same name
      ((or (and (> (length files) 1)
                (string= (nth 0 files) (nth 1 files)))
           (eq (length files) 1))
-      (ffip-find-files (nth 0 files)
+      (setq keyword (nth 0 files))
+      (when ffip-diff-find-file-by-file-name-p
+        (setq keyword (file-name-nondirectory keyword)))
+
+      (ffip-find-files keyword
                        open-another-window
                        nil
                        `(lambda (opened-file)
                           ;; use line number in new file since there
                           ;; is only one file name candidate
                           (ffip--forward-line ,blnum))))
+
      (t
-      ;; need pick a file
-      (run-hook-with-args 'ffip-diff-find-file-before-hook)
-      (ffip-find-files (cond
-                        ((string= (nth 0 files) "null")
-                         (nth 1 files))
-                        (t
-                         (nth 0 files)))
+      (unless (setq keyword (run-hook-with-args 'ffip-diff-find-file-before-hook
+                                                (nth 0 files)
+                                                (nth 1 files)))
+        ;; pick a file name from A and B
+        (setq keyword (cond
+                       ((string= (nth 0 files) "null")
+                        (nth 1 files))
+                       (t
+                        (nth 0 files)))))
+      (when ffip-diff-find-file-by-file-name-p
+        (setq keyword (file-name-nondirectory keyword)))
+
+      (ffip-find-files keyword
                        open-another-window
                        nil
                        (lambda (opened-file)
@@ -1161,15 +1294,12 @@ If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
 (defvar ffip-diff-mode-map
   (let ((map (make-sparse-keymap)))
     (set-keymap-parent map diff-mode-map)
-    ;; EVIL friendly. ffip-diff-mode is read-only
-    (define-key map "K" 'diff-hunk-prev)
-    (define-key map "J" 'diff-hunk-next)
-    (define-key map "P" 'diff-file-prev)
-    (define-key map "N" 'diff-file-next)
+    (define-key map (kbd "C-c C-k") 'ffip-diff-filter-hunks-by-file-name)
     (define-key map [remap diff-goto-source] 'ffip-diff-find-file)
     map)
   "Mode map based on `diff-mode-map'.")
 
+;;;###autoload
 (define-derived-mode ffip-diff-mode diff-mode "ffip"
   "Show diff/patch."
   (setq buffer-read-only t)
@@ -1200,7 +1330,7 @@ If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
       (cond
        ;; shell command
        ((stringp backend)
-        (ffip-show-content-in-diff-mode (ffip-shell-command-to-string backend)))
+        (ffip-show-content-in-diff-mode (shell-command-to-string backend)))
        ;; command
        ((functionp backend)
         (ffip-show-content-in-diff-mode (funcall backend)))
@@ -1270,6 +1400,7 @@ If NUM is not nil, the corresponding backend is executed directly."
          (if (string-match "^\\([0-9]+\\): " file)
              (ffip-show-diff-internal (string-to-number (match-string 1 file))))))))))
 
+;;;###autoload
 (defalias 'ffip-show-diff 'ffip-show-diff-by-description)
 
 (defadvice read-file-name (around ffip-read-file-name-hack activate)
@@ -1305,9 +1436,65 @@ If REVERSE is t, applied patch is reverted."
    (t
     (message "This command only run in `diff-mode' and `ffip-diff-mode'."))))
 
+(defun ffip-diff-hunk-file-name-match-p (keyword neg-keywords)
+  "Current hunk's file name does match KEYWORD and doesn't NEG-KEYWORDS."
+  (let* ((filenames (diff-hunk-file-names))
+         (f0 (nth 0 filenames))
+         (f1 (nth 1 filenames))
+         rlt)
+    (when (and filenames (> (length filenames) 1))
+      (setq rlt (or (string-match keyword f0) (string-match keyword f1)))
+
+      (let ((i 0) nk)
+        (while (and rlt (< i (length neg-keywords)))
+          (setq nk (nth i neg-keywords))
+          (setq rlt (not (or (string-match nk f0) (string-match nk f1))))
+          (setq i (1+ i)))))
+    rlt))
+
+;;;###autoload
+(defun ffip-diff-filter-hunks-by-file-name ()
+  "Filter hunks by file names which are generated from user input patterns.
+E.g., \"regex !exclude1 exclude1\" means the hunk's file name should
+match \"regex\", but should not match \"exclude1\" or \"exclude2\".
+Please note in \"regex\", space represents any string."
+  (interactive)
+  (cond
+   ((derived-mode-p 'diff-mode)
+    (let* ((pattern (read-string "File pattern (e.g., \"regex !exclude1 exclude2\"): "))
+           arr
+           keyword
+           neg-keywords
+           (first-hunk-position (save-excursion
+                                  (goto-char (point-min))
+                                  (re-search-forward diff-hunk-header-re)
+                                  (line-end-position))))
+      (cond
+       ((and pattern (not (string= pattern "")))
+        (setq arr (split-string pattern "!"))
+        (setq keyword (string-trim (nth 0 arr)))
+        (when (> (length arr) 1)
+          (setq neg-keywords (split-string (string-trim (nth 1 arr)) " +")))
+
+        ;; kill from the bottom to the top
+        (goto-char (point-max))
+        (condition-case nil
+            (while (not (and (<= (point) first-hunk-position)
+                             (ffip-diff-hunk-file-name-match-p keyword neg-keywords)))
+              (cond
+               ((not (ffip-diff-hunk-file-name-match-p keyword neg-keywords))
+                (diff-file-kill))
+               (t
+                (diff-file-prev))))
+          (error nil)))
+       (t
+        (message "File kill pattern should not be empty")))))
+   (t
+    (message "This command only run in `diff-mode' and `ffip-diff-mode'."))))
+
 ;;;###autoload
 (defun ffip-fix-file-path-at-point (&optional absolute-path-p)
-  "Fix file path at point.
+  "Fix file path at point.  File path could contain environment variables.
 If ABSOLUTE-PATH-P is t, old path is replaced by correct absolute path.
 Or else it's replaced by relative path."
   (interactive "P")
@@ -1323,7 +1510,10 @@ Or else it's replaced by relative path."
       nil)
 
      ;; find a file
-     ((setq cands (ffip-project-search (replace-regexp-in-string ffip-relative-path-pattern "" filename)))
+     ((setq cands
+            (ffip-project-search (replace-regexp-in-string ffip-relative-path-pattern
+                                                           ""
+                                                           filename)))
       (cond
        ((eq (length cands) 1)
         (setq full-path (nth 0 cands)))
